@@ -5,17 +5,16 @@ in astropy APE 14 (https://doi.org/10.5281/zenodo.1188875).
 
 """
 
-from astropy.wcs.wcsapi import BaseHighLevelWCS, BaseLowLevelWCS
-from astropy.modeling import separable
 import astropy.units as u
+from astropy.modeling import separable
+from astropy.wcs.wcsapi import BaseLowLevelWCS, HighLevelWCSMixin
 
-from . import utils
-from . import coordinate_frames as cf
+from gwcs import utils
 
 __all__ = ["GWCSAPIMixin"]
 
 
-class GWCSAPIMixin(BaseHighLevelWCS, BaseLowLevelWCS):
+class GWCSAPIMixin(BaseLowLevelWCS, HighLevelWCSMixin):
     """
     A mix-in class that is intended to be inherited by the
     :class:`~gwcs.wcs.WCS` class and provides the low- and high-level
@@ -52,14 +51,7 @@ class GWCSAPIMixin(BaseHighLevelWCS, BaseLowLevelWCS):
         arbitrary string.  Alternatively, if the physical type is
         unknown/undefined, an element can be `None`.
         """
-        # A CompositeFrame orders the output correctly based on axes_order.
-        if isinstance(self.output_frame, cf.CompositeFrame):
-            return self.output_frame.axis_physical_types
-
-        # If we don't have a CompositeFrame, where this is taken care of for us,
-        # we need to make sure we re-order the output to match the transform.
-        # The underlying frames don't reorder themselves because axes_order is global.
-        return tuple(self.output_frame.axis_physical_types[i] for i in self.output_frame.axes_order)
+        return self.output_frame.axis_physical_types
 
     @property
     def world_axis_units(self):
@@ -71,26 +63,23 @@ class GWCSAPIMixin(BaseHighLevelWCS, BaseLowLevelWCS):
         specification document, units that do not follow this standard are still
         allowed, but just not recommended).
         """
-        return tuple(unit.to_string(format='vounit') for unit in self.output_frame.unit)
+        return tuple(unit.to_string(format="vounit") for unit in self.output_frame.unit)
 
     def _remove_quantity_output(self, result, frame):
         if self.forward_transform.uses_quantity:
-            if self.output_frame.naxes == 1:
+            if frame.naxes == 1:
                 result = [result]
 
-            result = tuple(r.to_value(unit) for r, unit in zip(result, frame.unit))
+            result = tuple(
+                r.to_value(unit) if isinstance(r, u.Quantity) else r
+                for r, unit in zip(result, frame.unit, strict=False)
+            )
 
         # If we only have one output axes, we shouldn't return a tuple.
         if self.output_frame.naxes == 1 and isinstance(result, tuple):
             return result[0]
         return result
 
-    def _add_units_input(self, arrays, transform, frame):
-        if transform.uses_quantity:
-            return tuple(u.Quantity(array, unit) for array, unit in zip(arrays, frame.unit))
-
-        return arrays
-
     def pixel_to_world_values(self, *pixel_arrays):
         """
         Convert pixel coordinates to world coordinates.
@@ -104,18 +93,17 @@ class GWCSAPIMixin(BaseHighLevelWCS, BaseLowLevelWCS):
         order, where for an image, ``x`` is the horizontal coordinate and ``y``
         is the vertical coordinate.
         """
-        pixel_arrays = self._add_units_input(pixel_arrays, self.forward_transform, self.input_frame)
-        result = self(*pixel_arrays, with_units=False)
+        result = self._call_forward(*pixel_arrays)
 
         return self._remove_quantity_output(result, self.output_frame)
 
     def array_index_to_world_values(self, *index_arrays):
         """
         Convert array indices to world coordinates.
-        This is the same as `~BaseLowLevelWCS.pixel_to_world_values` except that
-        the indices should be given in ``(i, j)`` order, where for an image
+        This is the same as `~astropy.wcs.wcsapi.BaseLowLevelWCS.pixel_to_world_values`
+        except that the indices should be given in ``(i, j)`` order, where for an image
         ``i`` is the row and ``j`` is the column (i.e. the opposite order to
-        `~BaseLowLevelWCS.pixel_to_world_values`).
+        `~astropy.wcs.wcsapi.BaseLowLevelWCS.pixel_to_world_values`).
         """
         pixel_arrays = index_arrays[::-1]
         return self.pixel_to_world_values(*pixel_arrays)
@@ -132,32 +120,21 @@ class GWCSAPIMixin(BaseHighLevelWCS, BaseLowLevelWCS):
         be returned in the ``(x, y)`` order, where for an image, ``x`` is the
         horizontal coordinate and ``y`` is the vertical coordinate.
         """
-        try:
-            backward_transform = self.backward_transform
-            world_arrays = self._add_units_input(world_arrays,
-                                                 backward_transform,
-                                                 self.output_frame)
-        except NotImplementedError:
-            pass
-
-        result = self.invert(*world_arrays, with_units=False)
+        result = self._call_backward(*world_arrays)
 
         return self._remove_quantity_output(result, self.input_frame)
 
     def world_to_array_index_values(self, *world_arrays):
         """
         Convert world coordinates to array indices.
-        This is the same as `~BaseLowLevelWCS.world_to_pixel_values` except that
-        the indices should be returned in ``(i, j)`` order, where for an image
-        ``i`` is the row and ``j`` is the column (i.e. the opposite order to
-        `~BaseLowLevelWCS.pixel_to_world_values`). The indices should be
-        returned as rounded integers.
+        This is the same as `~astropy.wcs.wcsapi.BaseLowLevelWCS.world_to_pixel_values`
+        except that the indices should be returned in ``(i, j)`` order, where for an
+        image ``i`` is the row and ``j`` is the column (i.e. the opposite order to
+        `~astropy.wcs.wcsapi.BaseLowLevelWCS.pixel_to_world_values`). The indices should
+        be returned as rounded integers.
         """
         results = self.world_to_pixel_values(*world_arrays)
-        if self.pixel_n_dim == 1:
-            results = (results,)
-        else:
-            results = results[::-1]
+        results = (results,) if self.pixel_n_dim == 1 else results[::-1]
 
         results = tuple(utils._toindex(result) for result in results)
         return results[0] if self.pixel_n_dim == 1 else results
@@ -166,7 +143,7 @@ class GWCSAPIMixin(BaseHighLevelWCS, BaseLowLevelWCS):
     def array_shape(self):
         """
         The shape of the data that the WCS applies to as a tuple of
-        length `~BaseLowLevelWCS.pixel_n_dim`.
+        length `~astropy.wcs.wcsapi.BaseLowLevelWCS.pixel_n_dim`.
         If the WCS is valid in the context of a dataset with a particular
         shape, then this property can be used to store the shape of the
         data. This can be used for example if implementing slicing of WCS
@@ -177,8 +154,7 @@ class GWCSAPIMixin(BaseHighLevelWCS, BaseLowLevelWCS):
         """
         if self._pixel_shape is None:
             return None
-        else:
-            return self._pixel_shape[::-1]
+        return self._pixel_shape[::-1]
 
     @array_shape.setter
     def array_shape(self, value):
@@ -191,12 +167,13 @@ class GWCSAPIMixin(BaseHighLevelWCS, BaseLowLevelWCS):
     def pixel_bounds(self):
         """
         The bounds (in pixel coordinates) inside which the WCS is defined,
-        as a list with `~BaseLowLevelWCS.pixel_n_dim` ``(min, max)`` tuples.
-        The bounds should be given in ``[(xmin, xmax), (ymin, ymax)]``
-        order. WCS solutions are sometimes only guaranteed to be accurate
-        within a certain range of pixel values, for example when defining a
-        WCS that includes fitted distortions. This is an optional property,
-        and it should return `None` if a shape is not known or relevant.
+        as a list with `~astropy.wcs.wcsapi.BaseLowLevelWCS.pixel_n_dim`
+        ``(min, max)`` tuples.  The bounds should be given in
+        ``[(xmin, xmax), (ymin, ymax)]`` order. WCS solutions are sometimes
+        only guaranteed to be accurate within a certain range of pixel values,
+        for example when defining a WCS that includes fitted distortions. This
+        is an optional property, and it should return `None` if a shape is not
+        known or relevant.
         """
         bounding_box = self.bounding_box
         if bounding_box is None:
@@ -208,11 +185,7 @@ class GWCSAPIMixin(BaseHighLevelWCS, BaseLowLevelWCS):
         # Iterate over the bounding box and convert from quantity if required.
         bounding_box = list(bounding_box)
         for i, bb_axes in enumerate(bounding_box):
-            bb = []
-            for lim in bb_axes:
-                if isinstance(lim, u.Quantity):
-                    lim = lim.value
-                bb.append(lim)
+            bb = [lim.value if isinstance(lim, u.Quantity) else lim for lim in bb_axes]
 
             bounding_box[i] = tuple(bb)
 
@@ -241,21 +214,24 @@ class GWCSAPIMixin(BaseHighLevelWCS, BaseLowLevelWCS):
             return
         wcs_naxes = self.input_frame.naxes
         if len(value) != wcs_naxes:
-            raise ValueError("The number of data axes, "
-                             "{}, does not equal the "
-                             "shape {}.".format(wcs_naxes, len(value)))
+            msg = (
+                "The number of data axes, "
+                f"{wcs_naxes}, does not equal the "
+                f"shape {len(value)}."
+            )
+            raise ValueError(msg)
 
         self._pixel_shape = tuple(value)
 
     @property
     def axis_correlation_matrix(self):
         """
-        Returns an (`~BaseLowLevelWCS.world_n_dim`,
-        `~BaseLowLevelWCS.pixel_n_dim`) matrix that indicates using booleans
-        whether a given world coordinate depends on a given pixel coordinate.
-        This defaults to a matrix where all elements are `True` in the absence of
-        any further information. For completely independent axes, the diagonal
-        would be `True` and all other entries `False`.
+        Returns an (`~astropy.wcs.wcsapi.BaseLowLevelWCS.world_n_dim`,
+        `~astropy.wcs.wcsapi.BaseLowLevelWCS.pixel_n_dim`) matrix that indicates
+        using booleans whether a given world coordinate depends on a given pixel
+        coordinate.  This defaults to a matrix where all elements are `True` in
+        the absence of any further information. For completely independent axes,
+        the diagonal would be `True` and all other entries `False`.
         """
         return separable.separability_matrix(self.forward_transform)
 
@@ -269,77 +245,11 @@ class GWCSAPIMixin(BaseHighLevelWCS, BaseLowLevelWCS):
 
     @property
     def world_axis_object_classes(self):
-        return self.output_frame._world_axis_object_classes
+        return self.output_frame.world_axis_object_classes
 
     @property
     def world_axis_object_components(self):
-        return self.output_frame._world_axis_object_components
-
-    # High level APE 14 API
-
-    @property
-    def low_level_wcs(self):
-        """
-        Returns a reference to the underlying low-level WCS object.
-        """
-        return self
-
-    def _sanitize_pixel_inputs(self, *pixel_arrays):
-        pixels = []
-        if self.forward_transform.uses_quantity:
-            for i, pixel in enumerate(pixel_arrays):
-                if not isinstance(pixel, u.Quantity):
-                    pixel = u.Quantity(value=pixel, unit=self.input_frame.unit[i])
-                pixels.append(pixel)
-        else:
-            for i, pixel in enumerate(pixel_arrays):
-                if isinstance(pixel, u.Quantity):
-                    if pixel.unit != self.input_frame.unit[i]:
-                        raise ValueError('Quantity input does not match the '
-                                         'input_frame unit.')
-                    pixel = pixel.value
-                pixels.append(pixel)
-
-        return pixels
-
-    def pixel_to_world(self, *pixel_arrays):
-        """
-        Convert pixel values to world coordinates.
-        """
-        pixels = self._sanitize_pixel_inputs(*pixel_arrays)
-        return self(*pixels, with_units=True)
-
-    def array_index_to_world(self, *index_arrays):
-        """
-        Convert array indices to world coordinates (represented by Astropy
-        objects).
-        """
-        pixel_arrays = index_arrays[::-1]
-        pixels = self._sanitize_pixel_inputs(*pixel_arrays)
-        return self(*pixels, with_units=True)
-
-    def world_to_pixel(self, *world_objects):
-        """
-        Convert world coordinates to pixel values.
-        """
-        result = self.invert(*world_objects, with_units=True)
-        if self.input_frame.naxes > 1:
-            first_res = result[0]
-            if not utils.isnumerical(first_res):
-                result = [i.value for i in result]
-        else:
-            if not utils.isnumerical(result):
-                result = result.value
-
-        return result
-
-    def world_to_array_index(self, *world_objects):
-        """
-        Convert world coordinates (represented by Astropy objects) to array
-        indices.
-        """
-        result = self.invert(*world_objects, with_units=True)[::-1]
-        return tuple([utils._toindex(r) for r in result])
+        return self.output_frame.world_axis_object_components
 
     @property
     def pixel_axis_names(self):
@@ -348,7 +258,7 @@ class GWCSAPIMixin(BaseHighLevelWCS, BaseLowLevelWCS):
         """
         if self.input_frame is not None:
             return self.input_frame.axes_names
-        return tuple([''] * self.pixel_n_dim)
+        return tuple([""] * self.pixel_n_dim)
 
     @property
     def world_axis_names(self):
@@ -357,4 +267,4 @@ class GWCSAPIMixin(BaseHighLevelWCS, BaseLowLevelWCS):
         """
         if self.output_frame is not None:
             return self.output_frame.axes_names
-        return tuple([''] * self.world_n_dim)
+        return tuple([""] * self.world_n_dim)
