@@ -3,53 +3,50 @@
 Utility function for WCS
 
 """
-import re
+
 import functools
+import re
+
+import astropy.units as u
 import numpy as np
-from astropy.modeling import models as astmodels
-from astropy.modeling import core, projections
-from astropy.io import fits
 from astropy import coordinates as coords
-from astropy import units as u
-from astropy.time import Time, TimeDelta
-from astropy import table
+from astropy.io import fits
+from astropy.modeling import core, projections
+from astropy.modeling import models as astmodels
 from astropy.wcs import Celprm
 
-
 # these ctype values do not include yzLN and yzLT pairs
-sky_pairs = {"equatorial": ["RA", "DEC"],
-             "ecliptic": ["ELON", "ELAT"],
-             "galactic": ["GLON", "GLAT"],
-             "helioecliptic": ["HLON", "HLAT"],
-             "supergalactic": ["SLON", "SLAT"],
-             # "spec": specsystems
-            }
+sky_pairs = {
+    "equatorial": ["RA", "DEC"],
+    "ecliptic": ["ELON", "ELAT"],
+    "galactic": ["GLON", "GLAT"],
+    "helioecliptic": ["HLON", "HLAT"],
+    "supergalactic": ["SLON", "SLAT"],
+    # "spec": specsystems
+}
 
-radesys = ['ICRS', 'FK5', 'FK4', 'FK4-NO-E', 'GAPPT', 'GALACTIC']
+radesys = ["ICRS", "FK5", "FK4", "FK4-NO-E", "GAPPT", "GALACTIC"]
 
 
 class UnsupportedTransformError(Exception):
-
     def __init__(self, message):
-        super(UnsupportedTransformError, self).__init__(message)
+        super().__init__(message)
 
 
 class UnsupportedProjectionError(Exception):
     def __init__(self, code):
-        message = "Unsupported projection: {0}".format(code)
-        super(UnsupportedProjectionError, self).__init__(message)
+        message = f"Unsupported projection: {code}"
+        super().__init__(message)
 
 
 class RegionError(Exception):
-
     def __init__(self, message):
-        super(RegionError, self).__init__(message)
+        super().__init__(message)
 
 
 class CoordinateFrameError(Exception):
-
     def __init__(self, message):
-        super(CoordinateFrameError, self).__init__(message)
+        super().__init__(message)
 
 
 def _toindex(value):
@@ -71,8 +68,7 @@ def _toindex(value):
     >>> _toindex(np.array([1.5, 2.49999]))
     array([2, 2])
     """
-    indx = np.asarray(np.floor(np.asarray(value) + 0.5), dtype=int)
-    return indx
+    return np.asarray(np.floor(np.asarray(value) + 0.5), dtype=int)
 
 
 def get_values(units, *args):
@@ -88,7 +84,7 @@ def get_values(units, *args):
         Quantity inputs.
     """
     if units is not None:
-        result = [a.to_value(unit) for a, unit in zip(args, units)]
+        result = [a.to_value(unit) for a, unit in zip(args, units, strict=False)]
     else:
         result = [a.value for a in args]
     return result
@@ -158,9 +154,10 @@ def get_projcode(wcs_info):
     sky_axes, _, _ = get_axes(wcs_info)
     if not sky_axes:
         return None
-    projcode = wcs_info['CTYPE'][sky_axes[0]][5:8].upper()
+    projcode = wcs_info["CTYPE"][sky_axes[0]][5:8].upper()
     if projcode not in projections.projcodes:
-        raise UnsupportedProjectionError('Projection code %s, not recognized' % projcode)
+        msg = f"Projection code {projcode}, not recognized"
+        raise UnsupportedProjectionError(msg)
     return projcode
 
 
@@ -181,25 +178,23 @@ def read_wcs_from_header(header):
     wcs_info = {}
 
     try:
-        wcs_info['WCSAXES'] = header['WCSAXES']
+        wcs_info["WCSAXES"] = header["WCSAXES"]
     except KeyError:
-        p = re.compile(r'ctype[\d]*', re.IGNORECASE)
-        ctypes = header['CTYPE*']
+        p = re.compile(r"ctype[\d]*", re.IGNORECASE)
+        ctypes = header["CTYPE*"]
         keys = list(ctypes.keys())
         for key in keys[::-1]:
             if p.split(key)[-1] != "":
                 keys.remove(key)
-        wcs_info['WCSAXES'] = len(keys)
-    wcsaxes = wcs_info['WCSAXES']
+        wcs_info["WCSAXES"] = len(keys)
+    wcsaxes = wcs_info["WCSAXES"]
     # if not present call get_csystem
-    wcs_info['RADESYS'] = header.get('RADESYS', 'ICRS')
-    wcs_info['VAFACTOR'] = header.get('VAFACTOR', 1)
-    wcs_info['NAXIS'] = header.get('NAXIS', 0)
-    # date keyword?
-    # wcs_info['DATEOBS'] = header.get('DATE-OBS', 'DATEOBS')
-    wcs_info['EQUINOX'] = header.get("EQUINOX", None)
-    wcs_info['EPOCH'] = header.get("EPOCH", None)
-    wcs_info['DATEOBS'] = header.get("MJD-OBS", header.get("DATE-OBS", None))
+    wcs_info["RADESYS"] = header.get("RADESYS", "ICRS")
+    wcs_info["VAFACTOR"] = header.get("VAFACTOR", 1)
+    wcs_info["NAXIS"] = header.get("NAXIS", 0)
+    wcs_info["EQUINOX"] = header.get("EQUINOX", None)
+    wcs_info["EPOCH"] = header.get("EPOCH", None)
+    wcs_info["DATEOBS"] = header.get("MJD-OBS", header.get("DATE-OBS", None))
 
     ctype = []
     cunit = []
@@ -207,35 +202,32 @@ def read_wcs_from_header(header):
     crval = []
     cdelt = []
     for i in range(1, wcsaxes + 1):
-        ctype.append(header['CTYPE{0}'.format(i)])
-        cunit.append(header.get('CUNIT{0}'.format(i), None))
-        crpix.append(header.get('CRPIX{0}'.format(i), 0.0))
-        crval.append(header.get('CRVAL{0}'.format(i), 0.0))
-        cdelt.append(header.get('CDELT{0}'.format(i), 1.0))
+        ctype.append(header[f"CTYPE{i}"])
+        cunit.append(header.get(f"CUNIT{i}", None))
+        crpix.append(header.get(f"CRPIX{i}", 0.0))
+        crval.append(header.get(f"CRVAL{i}", 0.0))
+        cdelt.append(header.get(f"CDELT{i}", 1.0))
 
-    if 'CD1_1' in header:
-        wcs_info['has_cd'] = True
+    if "CD1_1" in header:
+        wcs_info["has_cd"] = True
     else:
-        wcs_info['has_cd'] = False
+        wcs_info["has_cd"] = False
     pc = np.zeros((wcsaxes, wcsaxes))
     for i in range(1, wcsaxes + 1):
         for j in range(1, wcsaxes + 1):
-            try:
-                if wcs_info['has_cd']:
-                    pc[i - 1, j - 1] = header['CD{0}_{1}'.format(i, j)]
-                else:
-                    pc[i - 1, j - 1] = header['PC{0}_{1}'.format(i, j)]
-            except KeyError:
-                if i == j:
-                    pc[i - 1, j - 1] = 1.
-                else:
-                    pc[i - 1, j - 1] = 0.
-    wcs_info['CTYPE'] = ctype
-    wcs_info['CUNIT'] = cunit
-    wcs_info['CRPIX'] = crpix
-    wcs_info['CRVAL'] = crval
-    wcs_info['CDELT'] = cdelt
-    wcs_info['PC'] = pc
+            key = f"CD{i}_{j}" if wcs_info["has_cd"] else f"PC{i}_{j}"
+            if key in header:
+                pc[i - 1, j - 1] = header[key]
+            elif i == j:
+                pc[i - 1, j - 1] = 1.0
+            else:
+                pc[i - 1, j - 1] = 0.0
+    wcs_info["CTYPE"] = ctype
+    wcs_info["CUNIT"] = cunit
+    wcs_info["CRPIX"] = crpix
+    wcs_info["CRVAL"] = crval
+    wcs_info["CDELT"] = cdelt
+    wcs_info["PC"] = pc
     return wcs_info
 
 
@@ -251,7 +243,7 @@ def get_axes(header):
     Returns
     -------
     sky_inmap, spectral_inmap, unknown : lists
-        indices in the input representing sky and spectral cordinates.
+        indices in the input representing sky and spectral coordinates.
 
     """
     if isinstance(header, fits.Header):
@@ -259,11 +251,12 @@ def get_axes(header):
     elif isinstance(header, dict):
         wcs_info = header
     else:
-        raise TypeError("Expected a FITS Header or a dict.")
+        msg = "Expected a FITS Header or a dict."
+        raise TypeError(msg)
 
     # Split each CTYPE value at "-" and take the first part.
     # This should represent the coordinate system.
-    ctype = [ax.split('-')[0].upper() for ax in wcs_info['CTYPE']]
+    ctype = [ax.split("-")[0].upper() for ax in wcs_info["CTYPE"]]
     sky_inmap = []
     spec_inmap = []
     unknown = []
@@ -284,32 +277,43 @@ def get_axes(header):
 
 
 def _is_skysys_consistent(ctype, sky_inmap):
-    """ Determine if the sky axes in CTYPE mathch to form a standard celestial system."""
+    """Determine if the sky axes in CTYPE match to form a standard celestial system."""
 
     for item in sky_pairs.values():
         if ctype[sky_inmap[0]] == item[0]:
             if ctype[sky_inmap[1]] != item[1]:
-                raise ValueError(
-                    "Inconsistent ctype for sky coordinates {0} and {1}".format(*ctype))
+                msg = "Inconsistent ctype for sky coordinates {} and {}".format(*ctype)
+                raise ValueError(msg)
             break
-        elif ctype[sky_inmap[1]] == item[0]:
+        if ctype[sky_inmap[1]] == item[0]:
             if ctype[sky_inmap[0]] != item[1]:
-                raise ValueError(
-                    "Inconsistent ctype for sky coordinates {0} and {1}".format(*ctype))
+                msg = "Inconsistent ctype for sky coordinates {} and {}".format(*ctype)
+                raise ValueError(msg)
             sky_inmap = sky_inmap[::-1]
             break
 
 
-specsystems = ["WAVE", "FREQ", "ENER", "WAVEN", "AWAV",
-               "VRAD", "VOPT", "ZOPT", "BETA", "VELO"]
+specsystems = [
+    "WAVE",
+    "FREQ",
+    "ENER",
+    "WAVEN",
+    "AWAV",
+    "VRAD",
+    "VOPT",
+    "ZOPT",
+    "BETA",
+    "VELO",
+]
 
-sky_systems_map = {'ICRS': coords.ICRS,
-                   'FK5': coords.FK5,
-                   'FK4': coords.FK4,
-                   'FK4NOE': coords.FK4NoETerms,
-                   'GAL': coords.Galactic,
-                   'HOR': coords.AltAz
-                  }
+sky_systems_map = {
+    "ICRS": coords.ICRS,
+    "FK5": coords.FK5,
+    "FK4": coords.FK4,
+    "FK4NOE": coords.FK4NoETerms,
+    "GAL": coords.Galactic,
+    "HOR": coords.AltAz,
+}
 
 
 def make_fitswcs_transform(header):
@@ -328,14 +332,15 @@ def make_fitswcs_transform(header):
     elif isinstance(header, dict):
         wcs_info = header
     else:
-        raise TypeError("Expected a FITS Header or a dict.")
+        msg = "Expected a FITS Header or a dict."
+        raise TypeError(msg)
     transforms = []
     wcs_linear = fitswcs_linear(wcs_info)
     transforms.append(wcs_linear)
     wcs_nonlinear = fitswcs_nonlinear(wcs_info)
     if wcs_nonlinear is not None:
         transforms.append(wcs_nonlinear)
-    return functools.reduce(core._model_oper('|'), transforms)
+    return functools.reduce(core._model_oper("|"), transforms)
 
 
 def fitswcs_linear(header):
@@ -353,9 +358,10 @@ def fitswcs_linear(header):
     elif isinstance(header, dict):
         wcs_info = header
     else:
-        raise TypeError("Expected a FITS Header or a dict.")
+        msg = "Expected a FITS Header or a dict."
+        raise TypeError(msg)
 
-    pc = wcs_info['PC']
+    pc = wcs_info["PC"]
     # get the part of the PC matrix corresponding to the imaging axes
     sky_axes, spec_axes, unknown = get_axes(wcs_info)
     if pc.shape != (2, 2):
@@ -375,28 +381,27 @@ def fitswcs_linear(header):
         crpix = []
         cdelt = []
         for i in sky_axes:
-            crpix.append(wcs_info['CRPIX'][i])
-            cdelt.append(wcs_info['CDELT'][i])
+            crpix.append(wcs_info["CRPIX"][i])
+            cdelt.append(wcs_info["CDELT"][i])
     else:
-        cdelt = wcs_info['CDELT']
-        crpix = wcs_info['CRPIX']
+        cdelt = wcs_info["CDELT"]
+        crpix = wcs_info["CRPIX"]
 
     # if wcsaxes == 2:
-    rotation = astmodels.AffineTransformation2D(matrix=pc, name='pc_matrix')
-    # elif wcsaxes == 3 :
-    # rotation = AffineTransformation3D(matrix=matrix)
-    # else:
-    # raise DimensionsError("WCSLinearTransform supports only 2 or 3 dimensions, "
-    # "{0} given".format(wcsaxes))
+    rotation = astmodels.AffineTransformation2D(matrix=pc, name="pc_matrix")
 
-    translation_models = [astmodels.Shift(-(shift - 1), name='crpix' + str(i + 1))
-                          for i, shift in enumerate(crpix)]
+    translation_models = [
+        astmodels.Shift(-(shift - 1), name="crpix" + str(i + 1))
+        for i, shift in enumerate(crpix)
+    ]
     translation = functools.reduce(lambda x, y: x & y, translation_models)
 
-    if not wcs_info['has_cd']:
+    if not wcs_info["has_cd"]:
         # Do not compute scaling since CDELT* = 1 if CD is present.
-        scaling_models = [astmodels.Scale(scale, name='cdelt' + str(i + 1))
-                          for i, scale in enumerate(cdelt)]
+        scaling_models = [
+            astmodels.Scale(scale, name="cdelt" + str(i + 1))
+            for i, scale in enumerate(cdelt)
+        ]
 
         scaling = functools.reduce(lambda x, y: x & y, scaling_models)
         wcs_linear = translation | rotation | scaling
@@ -420,7 +425,8 @@ def fitswcs_nonlinear(header):
     elif isinstance(header, dict):
         wcs_info = header
     else:
-        raise TypeError("Expected a FITS Header or a dict.")
+        msg = "Expected a FITS Header or a dict."
+        raise TypeError(msg)
 
     transforms = []
     projcode = get_projcode(wcs_info)
@@ -430,14 +436,14 @@ def fitswcs_nonlinear(header):
     # Create the sky rotation transform
     sky_axes, _, _ = get_axes(wcs_info)
     if sky_axes:
-        phip, lonp = [wcs_info['CRVAL'][i] for i in sky_axes]
+        phip, lonp = (wcs_info["CRVAL"][i] for i in sky_axes)
         # TODO: write "def compute_lonpole(projcode, l)"
-        # Set a defaul tvalue for now
+        # Set a default tvalue for now
         thetap = 180
         n2c = astmodels.RotateNative2Celestial(phip, lonp, thetap, name="crval")
         transforms.append(n2c)
     if transforms:
-        return functools.reduce(core._model_oper('|'), transforms)
+        return functools.reduce(core._model_oper("|"), transforms)
     return None
 
 
@@ -456,27 +462,42 @@ def create_projection_transform(projcode):
         Projection transform.
     """
 
-    projklassname = 'Pix2Sky_' + projcode
+    projklassname = f"Pix2Sky_{projcode}"
     try:
         projklass = getattr(projections, projklassname)
-    except AttributeError:
-        raise UnsupportedProjectionError(projcode)
+    except AttributeError as err:
+        raise UnsupportedProjectionError(projcode) from err
 
     projparams = {}
     return projklass(**projparams)
 
 
-def isnumerical(val):
+def is_high_level(*args, low_level_wcs):
     """
-    Determine if a value is numerical (number or np.array of numbers).
+    Determine if args matches the high level classes as defined by
+    ``low_level_wcs``.
     """
-    isnum = True
-    astropy_types=(coords.SkyCoord, u.Quantity, Time, TimeDelta, table.Column, table.Row)
-    if isinstance(val, astropy_types):
-        isnum = False
-    elif (isinstance(val, np.ndarray)
-          and not np.issubdtype(val.dtype, np.floating)
-          and not np.issubdtype(val.dtype, np.integer)
-          ):
-        isnum = False
-    return isnum
+    if len(args) != len(low_level_wcs.world_axis_object_classes):
+        return False
+
+    type_match = [
+        (type(arg), waoc[0])
+        for arg, waoc in zip(
+            args, low_level_wcs.world_axis_object_classes.values(), strict=False
+        )
+    ]
+
+    types_are_high_level = [argt is t for argt, t in type_match]
+
+    if all(types_are_high_level):
+        return True
+
+    if any(types_are_high_level):
+        msg = (
+            "Invalid types were passed, got "
+            f"({', '.join(tm[0].__name__ for tm in type_match)}) expected "
+            f"({', '.join(tm[1].__name__ for tm in type_match)})."
+        )
+        raise TypeError(msg)
+
+    return False
